/*
 * (C) Copyright 2002
 * Sysgo Real-Time Solutions, GmbH <www.elinos.com>
 * Marius Groeger <mgroeger@sysgo.de>
 *
 * (C) Copyright 2002
 * David Mueller, ELSOFT AG, <d.mueller@elsoft.ch>
 *
 * (C) Copyright 2003
 * Texas Instruments, <www.ti.com>
 * Kshitij Gupta <Kshitij@ti.com>
 *
 * (C) Copyright 2004
 * ARM Ltd.
 * Philippe Robin, <philippe.robin@arm.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <netdev.h>
#include <asm/arch/ast_scu.h>
#include <asm/arch/ast-sdmc.h>
#include <asm/io.h>

DECLARE_GLOBAL_DATA_PTR;

extern int ftgmac100_phy_read(struct eth_device *dev, int addr, int reg, u16 *value);
extern int ftgmac100_phy_write(struct eth_device *dev, int addr, int reg, u16 value);

#if defined(CONFIG_SHOW_BOOT_PROGRESS)
void show_boot_progress(int progress)
{
    printf("Boot reached stage %d\n", progress);
}
#endif

int board_init(void)
{
	/* adress of boot parameters */
	gd->bd->bi_boot_params = CONFIG_SYS_SDRAM_BASE + 0x100;
	gd->flags = 0;
	return 0;
}

int dram_init(void)
{
	/* dram_init must store complete ramsize in gd->ram_size */
	u32 vga = ast_scu_get_vga_memsize();
	u32 dram = ast_sdmc_get_mem_size();
	gd->ram_size = (dram - vga);

	return 0;
}

#ifdef CONFIG_FTGMAC100
int board_eth_init(bd_t *bd)
{
    int status; 
    u16 value;
    struct eth_device *ftgmac1_eth_device;

    status = ftgmac100_initialize(bd);

    if (status == 0)
    {
        printf("ftgmac100_initialize() failed - propagate error\n");
        return 0;
    }
    else
    {
        /* Customize Marvell PHY settings for Blue Whale BMC */

        ftgmac1_eth_device = eth_get_dev_by_name("FTGMAC100#1");

        status = ftgmac100_phy_write(ftgmac1_eth_device, 0, 0x16, 2);
        if (status == -1)
        {
            printf("%s: PHY write of Reg 0x16 to 2 failed\n",__FUNCTION__);
            return 0;
        }

        status = ftgmac100_phy_read(ftgmac1_eth_device, 0, 0x15, &value);
        if (status == -1)
        {
            printf("%s: PHY read of Reg 0x15 failed\n",__FUNCTION__);
            return 0;
        }

        /* Turn off bits 4 and 5 in Page 2 Register 21 */
        value &= ~0x30;

        status = ftgmac100_phy_write(ftgmac1_eth_device, 0, 0x15, value);
        if (status == -1)
        {
            printf("%s: PHY write of Reg 0x15 to %x failed\n",__FUNCTION__,value);
            return 0;
        }

        status = ftgmac100_phy_write(ftgmac1_eth_device, 0, 0x16, 0);
        if (status == -1)
        {
            printf("%s: PHY write of Reg 0x16 to 0 failed\n",__FUNCTION__);
            return 0;
        }
        
        status = ftgmac100_phy_read(ftgmac1_eth_device, 0, 0, &value);
        if (status == -1)
        {
            printf("%s: PHY read of Reg 0 failed\n",__FUNCTION__);
            return 0;
        }

        value |= 0x8000;

        status = ftgmac100_phy_write(ftgmac1_eth_device, 0, 0, value);
        if (status == -1)
        {
            printf("%s: PHY write of Reg 0 to %x failed\n",__FUNCTION__,value);
            return 0;
        }
        
        /* Return success */
        return 1;
    }
}
#endif

/*
 * Called by macro WATCHDOG_RESET
 */
#if defined(CONFIG_HW_WATCHDOG)

void hw_watchdog_reset (void)
{
    u32 reg=0;
	/*restart WD2 timer*/
    reg = 0x4755;
    writel(reg, AST_WDT_BASE | 0x28);
}

#endif /* CONFIG_WATCHDOG */
