/*
 * Hardware monitoring driver for TI TPS53915
 *
 * Copyright (c) 2017 Mellanox Technologies.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 */

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/err.h>
#include <linux/i2c.h>
#include "pmbus.h"

static struct pmbus_driver_info tps53915_info = {
	.pages = 1,
	/*
	 * *TBD* It is not clear whether the data format is linear or direct.
	 * Must be updated accordingly; either set the data format array or
	 * implement a device specific 'identify' callback.
	 */
	.func[0] = PMBUS_HAVE_STATUS_INPUT | PMBUS_HAVE_VIN |
			PMBUS_HAVE_STATUS_VOUT | PMBUS_HAVE_VOUT |
			PMBUS_HAVE_STATUS_TEMP | PMBUS_HAVE_TEMP |
			PMBUS_HAVE_STATUS_IOUT | PMBUS_HAVE_IOUT,
};

static int tps53915_probe(struct i2c_client *client,
			  const struct i2c_device_id *id)
{
	return pmbus_do_probe(client, id, &tps53915_info);
}

static const struct i2c_device_id tps53915_id[] = {
	{"tps53915", 0},
	{}
};
MODULE_DEVICE_TABLE(i2c, tps53915_id);

static const struct of_device_id tps53915_of_match[] = {
	{.compatible = "ti,tps53915",},
};
MODULE_DEVICE_TABLE(of, tps53915_of_match);

/* This is the driver that will be inserted */
static struct i2c_driver tps53915_driver = {
	.driver = {
			.name = "tps53915",
			.of_match_table = of_match_ptr(tps53915_of_match),
		   },
	.probe = tps53915_probe,
	.remove = pmbus_do_remove,
	.id_table = tps53915_id,
};

module_i2c_driver(tps53915_driver);

MODULE_AUTHOR("Mellanox Technologies");
MODULE_DESCRIPTION("PMBus driver for TI TPS53915");
MODULE_LICENSE("GPL");
